@@ -32,6 +32,8 @@ import (
 	"github.com/spf13/pflag"
 
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
+	"k8s.io/apiserver/pkg/server/mux"
+	"k8s.io/apiserver/pkg/server/routes"
 	"k8s.io/autoscaler/cluster-autoscaler/cloudprovider"
 	cloudBuilder "k8s.io/autoscaler/cluster-autoscaler/cloudprovider/builder"
 	"k8s.io/autoscaler/cluster-autoscaler/config"
@@ -161,17 +163,14 @@ var (
 	nodeAutoprovisioningEnabled      = flag.Bool("node-autoprovisioning-enabled", false, "Should CA autoprovision node groups when needed")
 	maxAutoprovisionedNodeGroupCount = flag.Int("max-autoprovisioned-node-group-count", 15, "The maximum number of autoprovisioned groups in the cluster.")
 
-	unremovableNodeRecheckTimeout       = flag.Duration("unremovable-node-recheck-timeout", 5*time.Minute, "The timeout before we check again a node that couldn't be removed before")
-	expendablePodsPriorityCutoff        = flag.Int("expendable-pods-priority-cutoff", -10, "Pods with priority below cutoff will be expendable. They can be killed without any consideration during scale down and they don't cause scale up. Pods with null priority (PodPriority disabled) are non expendable.")
-	regional                            = flag.Bool("regional", false, "Cluster is regional.")
-	newPodScaleUpDelay                  = flag.Duration("new-pod-scale-up-delay", 0*time.Second, "Pods less than this old will not be considered for scale-up.")
-	filterOutSchedulablePodsUsesPacking = flag.Bool("filter-out-schedulable-pods-uses-packing", true,
-		"Filtering out schedulable pods before CA scale up by trying to pack the schedulable pods on free capacity on existing nodes."+
-			"Setting it to false employs a more lenient filtering approach that does not try to pack the pods on the nodes."+
-			"Pods with nominatedNodeName set are always filtered out.")
+	unremovableNodeRecheckTimeout = flag.Duration("unremovable-node-recheck-timeout", 5*time.Minute, "The timeout before we check again a node that couldn't be removed before")
+	expendablePodsPriorityCutoff  = flag.Int("expendable-pods-priority-cutoff", -10, "Pods with priority below cutoff will be expendable. They can be killed without any consideration during scale down and they don't cause scale up. Pods with null priority (PodPriority disabled) are non expendable.")
+	regional                      = flag.Bool("regional", false, "Cluster is regional.")
+	newPodScaleUpDelay            = flag.Duration("new-pod-scale-up-delay", 0*time.Second, "Pods less than this old will not be considered for scale-up.")
 
 	ignoreTaintsFlag         = multiStringFlag("ignore-taint", "Specifies a taint to ignore in node templates when considering to scale a node group")
 	awsUseStaticInstanceList = flag.Bool("aws-use-static-instance-list", false, "Should CA fetch instance types in runtime or use a static list. AWS only")
+	enableProfiling          = flag.Bool("profiling", false, "Is debug/pprof endpoint enabled")
 )
 
 func createAutoscalingOptions() config.AutoscalingOptions {
@@ -192,54 +191,53 @@ func createAutoscalingOptions() config.AutoscalingOptions {
 		klog.Fatalf("Failed to parse flags: %v", err)
 	}
 	return config.AutoscalingOptions{
-		CloudConfig:                         *cloudConfig,
-		CloudProviderName:                   *cloudProviderFlag,
-		NodeGroupAutoDiscovery:              *nodeGroupAutoDiscoveryFlag,
-		MaxTotalUnreadyPercentage:           *maxTotalUnreadyPercentage,
-		OkTotalUnreadyCount:                 *okTotalUnreadyCount,
-		ScaleUpFromZero:                     *scaleUpFromZero,
-		EstimatorName:                       *estimatorFlag,
-		ExpanderName:                        *expanderFlag,
-		IgnoreDaemonSetsUtilization:         *ignoreDaemonSetsUtilization,
-		IgnoreMirrorPodsUtilization:         *ignoreMirrorPodsUtilization,
-		MaxBulkSoftTaintCount:               *maxBulkSoftTaintCount,
-		MaxBulkSoftTaintTime:                *maxBulkSoftTaintTime,
-		MaxEmptyBulkDelete:                  *maxEmptyBulkDeleteFlag,
-		MaxGracefulTerminationSec:           *maxGracefulTerminationFlag,
-		MaxNodeProvisionTime:                *maxNodeProvisionTime,
-		MaxNodesTotal:                       *maxNodesTotal,
-		MaxCoresTotal:                       maxCoresTotal,
-		MinCoresTotal:                       minCoresTotal,
-		MaxMemoryTotal:                      maxMemoryTotal,
-		MinMemoryTotal:                      minMemoryTotal,
-		GpuTotal:                            parsedGpuTotal,
-		NodeGroups:                          *nodeGroupsFlag,
-		ScaleDownDelayAfterAdd:              *scaleDownDelayAfterAdd,
-		ScaleDownDelayAfterDelete:           *scaleDownDelayAfterDelete,
-		ScaleDownDelayAfterFailure:          *scaleDownDelayAfterFailure,
-		ScaleDownEnabled:                    *scaleDownEnabled,
-		ScaleDownUnneededTime:               *scaleDownUnneededTime,
-		ScaleDownUnreadyTime:                *scaleDownUnreadyTime,
-		ScaleDownUtilizationThreshold:       *scaleDownUtilizationThreshold,
-		ScaleDownGpuUtilizationThreshold:    *scaleDownGpuUtilizationThreshold,
-		ScaleDownNonEmptyCandidatesCount:    *scaleDownNonEmptyCandidatesCount,
-		ScaleDownCandidatesPoolRatio:        *scaleDownCandidatesPoolRatio,
-		ScaleDownCandidatesPoolMinCount:     *scaleDownCandidatesPoolMinCount,
-		WriteStatusConfigMap:                *writeStatusConfigMapFlag,
-		BalanceSimilarNodeGroups:            *balanceSimilarNodeGroupsFlag,
-		ConfigNamespace:                     *namespace,
-		ClusterName:                         *clusterName,
-		NodeAutoprovisioningEnabled:         *nodeAutoprovisioningEnabled,
-		MaxAutoprovisionedNodeGroupCount:    *maxAutoprovisionedNodeGroupCount,
-		UnremovableNodeRecheckTimeout:       *unremovableNodeRecheckTimeout,
-		ExpendablePodsPriorityCutoff:        *expendablePodsPriorityCutoff,
-		Regional:                            *regional,
-		NewPodScaleUpDelay:                  *newPodScaleUpDelay,
-		FilterOutSchedulablePodsUsesPacking: *filterOutSchedulablePodsUsesPacking,
-		IgnoredTaints:                       *ignoreTaintsFlag,
-		NodeDeletionDelayTimeout:            *nodeDeletionDelayTimeout,
-		AWSUseStaticInstanceList:            *awsUseStaticInstanceList,
-		KubeConfigPath:                      *kubeConfigFile,
+		CloudConfig:                      *cloudConfig,
+		CloudProviderName:                *cloudProviderFlag,
+		NodeGroupAutoDiscovery:           *nodeGroupAutoDiscoveryFlag,
+		MaxTotalUnreadyPercentage:        *maxTotalUnreadyPercentage,
+		OkTotalUnreadyCount:              *okTotalUnreadyCount,
+		ScaleUpFromZero:                  *scaleUpFromZero,
+		EstimatorName:                    *estimatorFlag,
+		ExpanderName:                     *expanderFlag,
+		IgnoreDaemonSetsUtilization:      *ignoreDaemonSetsUtilization,
+		IgnoreMirrorPodsUtilization:      *ignoreMirrorPodsUtilization,
+		MaxBulkSoftTaintCount:            *maxBulkSoftTaintCount,
+		MaxBulkSoftTaintTime:             *maxBulkSoftTaintTime,
+		MaxEmptyBulkDelete:               *maxEmptyBulkDeleteFlag,
+		MaxGracefulTerminationSec:        *maxGracefulTerminationFlag,
+		MaxNodeProvisionTime:             *maxNodeProvisionTime,
+		MaxNodesTotal:                    *maxNodesTotal,
+		MaxCoresTotal:                    maxCoresTotal,
+		MinCoresTotal:                    minCoresTotal,
+		MaxMemoryTotal:                   maxMemoryTotal,
+		MinMemoryTotal:                   minMemoryTotal,
+		GpuTotal:                         parsedGpuTotal,
+		NodeGroups:                       *nodeGroupsFlag,
+		ScaleDownDelayAfterAdd:           *scaleDownDelayAfterAdd,
+		ScaleDownDelayAfterDelete:        *scaleDownDelayAfterDelete,
+		ScaleDownDelayAfterFailure:       *scaleDownDelayAfterFailure,
+		ScaleDownEnabled:                 *scaleDownEnabled,
+		ScaleDownUnneededTime:            *scaleDownUnneededTime,
+		ScaleDownUnreadyTime:             *scaleDownUnreadyTime,
+		ScaleDownUtilizationThreshold:    *scaleDownUtilizationThreshold,
+		ScaleDownGpuUtilizationThreshold: *scaleDownGpuUtilizationThreshold,
+		ScaleDownNonEmptyCandidatesCount: *scaleDownNonEmptyCandidatesCount,
+		ScaleDownCandidatesPoolRatio:     *scaleDownCandidatesPoolRatio,
+		ScaleDownCandidatesPoolMinCount:  *scaleDownCandidatesPoolMinCount,
+		WriteStatusConfigMap:             *writeStatusConfigMapFlag,
+		BalanceSimilarNodeGroups:         *balanceSimilarNodeGroupsFlag,
+		ConfigNamespace:                  *namespace,
+		ClusterName:                      *clusterName,
+		NodeAutoprovisioningEnabled:      *nodeAutoprovisioningEnabled,
+		MaxAutoprovisionedNodeGroupCount: *maxAutoprovisionedNodeGroupCount,
+		UnremovableNodeRecheckTimeout:    *unremovableNodeRecheckTimeout,
+		ExpendablePodsPriorityCutoff:     *expendablePodsPriorityCutoff,
+		Regional:                         *regional,
+		NewPodScaleUpDelay:               *newPodScaleUpDelay,
+		IgnoredTaints:                    *ignoreTaintsFlag,
+		KubeConfigPath:                   *kubeConfigFile,
+		NodeDeletionDelayTimeout:         *nodeDeletionDelayTimeout,
+		AWSUseStaticInstanceList:         *awsUseStaticInstanceList,
 	}
 }
 
@@ -369,9 +367,16 @@ func main() {
 	klog.V(1).Infof("Cluster Autoscaler %s", version.ClusterAutoscalerVersion)
 
 	go func() {
-		http.Handle("/metrics", legacyregistry.Handler())
-		http.Handle("/health-check", healthCheck)
-		err := http.ListenAndServe(*address, nil)
+		pathRecorderMux := mux.NewPathRecorderMux("cluster-autoscaler")
+		defaultMetricsHandler := legacyregistry.Handler().ServeHTTP
+		pathRecorderMux.HandleFunc("/metrics", func(w http.ResponseWriter, req *http.Request) {
+			defaultMetricsHandler(w, req)
+		})
+		pathRecorderMux.HandleFunc("/health-check", healthCheck.ServeHTTP)
+		if *enableProfiling {
+			routes.Profiling{}.Install(pathRecorderMux)
+		}
+		err := http.ListenAndServe(*address, pathRecorderMux)
 		klog.Fatalf("Failed to start metrics: %v", err)
 	}()
 
@@ -386,7 +391,7 @@ func main() {
 		kubeClient := createKubeClient(getKubeConfig())
 
 		// Validate that the client is ok.
-		_, err = kubeClient.CoreV1().Nodes().List(metav1.ListOptions{})
+		_, err = kubeClient.CoreV1().Nodes().List(ctx.TODO(), metav1.ListOptions{})
 		if err != nil {
 			klog.Fatalf("Failed to get nodes from apiserver: %v", err)
 		}
